/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.auth

import android.app.Activity
import android.content.Intent
import android.content.ServiceConnection
import android.net.Uri
import android.os.Bundle
import android.os.ResultReceiver
import androidx.appcompat.app.AppCompatActivity
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.util.KakaoCustomTabsClient
import com.kakao.sdk.common.util.SdkLog

/**
 * @suppress
 */
class AuthCodeHandlerActivity : AppCompatActivity() {
    private lateinit var resultReceiver: ResultReceiver
    private lateinit var fullUri: Uri
    private var customTabsConnection: ServiceConnection? = null
    private var customTabsOpened = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        try {
            intent.extras?.getBundle(Constants.KEY_BUNDLE)
                ?.run {
                    resultReceiver = getParcelable<ResultReceiver>(Constants.KEY_RESULT_RECEIVER) as ResultReceiver
                    fullUri = getParcelable<Uri>(Constants.KEY_FULL_URI) as Uri
                }
        } catch (e: Throwable) {
            SdkLog.e(e)
            sendError(ClientError(ClientErrorCause.Unknown).apply { initCause(e) })
        }
    }

    override fun onResume() {
        super.onResume()

        if (!customTabsOpened) {
            customTabsOpened = true
            if (this::fullUri.isInitialized) {
                openChromeCustomTab(fullUri)
            } else {
                sendError(ClientError(ClientErrorCause.IllegalState, "/oauth/authorize url has been not initialized."))
            }
        } else {
            sendError(ClientError(ClientErrorCause.Cancelled, "/oauth/authorize cancelled."))
        }
    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)

        intent?.data?.let {
            sendOK(it)
        } ?: {
            sendError(ClientError(ClientErrorCause.Cancelled, "/oauth/authorize cancelled."))
        } ()
    }

    private fun openChromeCustomTab(uri: Uri) {
        SdkLog.i("Authorize Uri: $uri")
        try {
            customTabsConnection = KakaoCustomTabsClient.openWithDefault(this, uri)
        } catch (e: UnsupportedOperationException) {
            SdkLog.w(e)
            KakaoCustomTabsClient.open(this, uri)
        }
    }

    private fun sendError(exception: KakaoSdkError) {
        if (this::resultReceiver.isInitialized) {
            resultReceiver.send(
                Activity.RESULT_CANCELED,
                Bundle().apply { putSerializable(Constants.KEY_EXCEPTION, exception) }
            )
        }
        finish()
    }

    private fun sendOK(uri: Uri) {
        if (this::resultReceiver.isInitialized) {
            resultReceiver.send(
                Activity.RESULT_OK,
                Bundle().apply { putParcelable(Constants.KEY_URL, uri) }
            )
        }
        finish()
    }

    override fun onDestroy() {
        super.onDestroy()
        customTabsConnection?.let { unbindService(it) }
    }
}