/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
@file:JvmName("UserApiClientKt")

package com.kakao.sdk.user

import com.kakao.sdk.auth.TokenManagerProvider
import com.kakao.sdk.auth.network.RxAuthOperations
import com.kakao.sdk.auth.network.rxKapiWithOAuth
import com.kakao.sdk.common.network.ApiFactory
import com.kakao.sdk.common.network.RxOperations
import com.kakao.sdk.user.model.AccessTokenInfo
import com.kakao.sdk.user.model.User
import com.kakao.sdk.user.model.UserServiceTerms
import com.kakao.sdk.user.model.UserShippingAddresses
import io.reactivex.Completable
import io.reactivex.Single
import java.util.*

/**
 * 사용자관리 API 호출을 담당하는 클라이언트. (for ReactiveX)
 */
class RxUserApiClient(
    private val userApi: RxUserApi = ApiFactory.rxKapiWithOAuth.create(RxUserApi::class.java),
    val authOperations: RxAuthOperations = RxAuthOperations.instance,
    private val tokenManagerProvider: TokenManagerProvider = TokenManagerProvider.instance
) {

    /**
     * 사용자에 대한 다양한 정보 획득.
     */
    @JvmOverloads
    fun me(secureReSource: Boolean = true): Single<User> =
        userApi.me(secureReSource)
            .compose(RxOperations.handleApiError())
            .compose(authOperations.handleApiError())

    /**
     * 현재 로그인한 사용자의 엑세스 토큰 정보 보기.
     *
     * [me] 에서 제공되는 다양한 사용자 정보 없이 가볍게 토큰의 유효성을 체크하는 용도로 사용하는 경우 추천합니다.
     * 액세스토큰이 만려되어있는 경우 리프레시토큰으로 갱신된 새로운 액세스토큰의 정보를 반환합니다.
     */
    fun accessTokenInfo(): Single<AccessTokenInfo> =
        userApi.accessTokenInfo()
            .compose(RxOperations.handleApiError())
            .compose(authOperations.handleApiError())

    /**
     * User 클래스에서 제공되고 있는 사용자의 부가정보를 신규저장 및 수정.
     *
     * 저장 가능한 키 이름은 카카오 디벨로퍼스 > 카카오 로그인 > 사용자 프로퍼티 메뉴에서 확인.
     * 앱 연결 시 기본 저장되는 nickname, profile_image, thumbnail_image 값도 덮어쓰기 가능하며 새로운 컬럼을 추가하면 해당 키 이름으로 정보 저장 가능.
     */
    fun updateProfile(properties: Map<String, String>): Completable =
        userApi.updateProfile(properties)
            .compose(RxOperations.handleCompletableError())
            .compose(authOperations.handleCompletableError())

    /**
     * 현재 토큰을 강제로 만료시키고 로그아웃.
     *
     * API 호출 결과와 관계 없이 [TokenManagerProvider]에 지정된 저장소에서 토큰을 자동으로 삭제함.
     */
    fun logout(): Completable =
        userApi.logout()
            .compose(RxOperations.handleCompletableError())
            .compose(authOperations.handleCompletableError())
            .doOnEvent { tokenManagerProvider.manager.clear() }

    /**
     * 연결 끊기. 카카오 로그인을 통한 사용자와 서비스 간의 연결 관계를 해제하고 사용자의 정보 제공 및 카카오 플랫폼 사용을 중단.
     *
     * API 호출에 성공하면 [TokenManagerProvider]에 지정된 저장소에서 토큰을 자동으로 삭제함.
     */
    fun unlink(): Completable =
        userApi.unlink()
            .compose(RxOperations.handleCompletableError())
            .compose(authOperations.handleCompletableError())
            .doOnComplete { tokenManagerProvider.manager.clear() }

    /**
     * 사용자의 배송지 정보 획득.
     *
     * @param fromUpdateAt
     * @param pageSize
     */
    @JvmOverloads
    fun shippingAddresses(
        fromUpdateAt: Date? = null,
        pageSize: Int? = null
    ): Single<UserShippingAddresses> {
        return userApi.shippingAddresses(fromUpdateAt = fromUpdateAt, pageSize = pageSize)
            .compose(RxOperations.handleApiError())
            .compose(authOperations.handleApiError())
    }

    /**
     * 사용자의 배송지 정보 획득.
     *
     * @param addressId 가져올 배송지 id
     */
    fun shippingAddresses(addressId: Long): Single<UserShippingAddresses> {
        return userApi.shippingAddresses(addressId)
            .compose(RxOperations.handleApiError())
            .compose(authOperations.handleApiError())
    }

    /**
     * 사용자가 카카오 간편가입을 통해 동의한 서비스 약관 내역 반환.
     */
    fun serviceTerms(): Single<UserServiceTerms> {
        return userApi.serviceTerms()
            .compose(RxOperations.handleApiError())
            .compose(authOperations.handleApiError())
    }

    companion object {
        /**
         * User API 를 호출하기 위한 rx singleton
         */
        @JvmStatic
        val instance by lazy { UserApiClient.rx }
    }
}

/**
 * ReactiveX 를 위한 [UserApiClient] singleton 객체
 */
val UserApiClient.Companion.rx by lazy { RxUserApiClient() }